/******************************************************************************
Header name: ckokey.h     Rev: 01  Date: 07-Dec-89 Programmer: C.P.Armstrong

Header title:   Definitions of key press combinations not provided for by the
                Presentation Manager.

Description:    Alt- key combinations do not produce unique key codes under the
                PM.  The keypad keys with NumLock off do not produce
                unique codes as defined under the PM.  Ctrl- and
                shift- function keys do not produce unique key codes
                as defined under the PM.  Alt- function key commands
                are intercepted by the PM and not provided as WM_CHAR
                messages.
                
                So I've invented by own key codes.
                If the Alt- key is pressed then bit 8 is set in the
                keys normal scan code.  If the Ctrl- key is pressed
                for a function key or direction key then bit 7 is set
                in the scan code. 
                
                To distiguish between the Numeric keypad direction codes and
                the grey direction keys the scan codes for the keypad keys must
                be filtered before testing for a valid virtual key code.  To
                allow the numeric characters generated by the keypad with
                NumLock on to pass a valid char code must be checked for first.
                

Modification History:
    01  07-Dec-89   C.P.Armstrong   created

******************************************************************************/
#define KPSLASH     0x5C
#define KPSTAR      0x37

#define KP7         0x47
#define KP8         0x48
#define KP9         0x49
#define KPMINUS     0x4A
#define KP4         0x4B
#define KP5         0x4C
#define KP6         0x4D
#define KPPLUS      0x4E
#define KP1         0x4F
#define KP2         0x50
#define KP3         0x51
#define KP0         0x52
#define KPDOT       0x53

#define ALT_1       0x82
#define ALT_2       0x83
#define ALT_3       0x84
#define ALT_4       0x85
#define ALT_5       0x86
#define ALT_6       0x87
#define ALT_7       0x88
#define ALT_8       0x89
#define ALT_9       0x8A
#define ALT_0       0x8B
#define ALT_MINUS   0x8C
#define ALT_EQUAL   0x8D
#define ALT_Q       0x90
#define ALT_W       0x91
#define ALT_E       0x92
#define ALT_R       0x93
#define ALT_T       0x94
#define ALT_Y       0x95
#define ALT_U       0x96
#define ALT_I       0x97
#define ALT_O       0x98
#define ALT_P       0x99
#define ALT_LSQ     0x9A
#define ALT_RSQ     0x9B
#define ALT_A       0x9E
#define ALT_S       0x9F
#define ALT_D       0xA0
#define ALT_F       0xA1
#define ALT_G       0xA2
#define ALT_H       0xA3
#define ALT_J       0xA4
#define ALT_K       0xA5
#define ALT_L       0xA6
#define ALT_SEMICOLON 0xA27
#define ALT_QUOTE   0xA8
#define ALT_Z       0xAC
#define ALT_X       0xAD
#define ALT_C       0xAE
#define ALT_V       0xAF
#define ALT_B       0xB0
#define ALT_N       0xB1
#define ALT_M       0xB2
#define ALT_COMMA   0xB3
#define ALT_PERIOD  0xB4
#define ALT_SLASH   0xB5
#define ALT_BSLASH  0xAB

#define VK_SHPRINTSCRN (VK_PRINTSCRN | 0x20)
